@@ -32,7 +32,14 @@ log = logging.getLogger("jjm")
 class JenkinsJobManager:
     """main jjb manager"""
 
-    __slots__ = ("config", "plugins_list", "_jenkins", "jobs", "views")
+    __slots__ = (
+        "config",
+        "plugins_list",
+        "_jenkins",
+        "jobs",
+        "views",
+        "validation_errors",
+    )
     job_managing_job_classes = frozenset(["jenkins.branch.OrganizationFolder"])
     raw_xml_yaml_path = "./raw_xml_jobs.yaml"
 
@@ -44,6 +51,7 @@ class JenkinsJobManager:
         self.plugins_list: Optional[list] = None
         self.jobs: Dict[str, XmlChange] = XmlChangeDefaultDict()
         self.views: Dict[str, XmlChange] = XmlChangeDefaultDict()
+        self.validation_errors = []
 
     @property
     def jenkins(self):
@@ -57,6 +65,7 @@ class JenkinsJobManager:
         return self._jenkins
 
     def check_authentication(self):
+        """check if jenkins connection config correct"""
         log.debug("checking credentials")
         try:
             self.jenkins.run_script("println 'ok'")
@@ -66,6 +75,7 @@ class JenkinsJobManager:
         return True
 
     def read_jobs(self):
+        """read existing jobs from jenkins"""
         jenkins = self.jenkins
         jobs = self.jobs
 
@@ -112,7 +122,10 @@ class JenkinsJobManager:
         return JJBConfig
 
     def jenkins_format_xml(self, xml_job: XmlJob):
-        """bounces job config through jenkins to get the formatting right"""
+        """
+        bounces job config through jenkins to get the formatting right
+        unused, deprecated
+        """
         jenkins = self.jenkins
         _xml: xml.etree.ElementTree.Element = xml_job.xml
 
@@ -139,7 +152,7 @@ class JenkinsJobManager:
         return formatted_xml
 
     def generate_jjb_xml(self):
-        """borrow jjb rendering to render jjb yaml to xml"""
+        """render jjb yaml to xml"""
 
         class RawXmlProject(jenkins_jobs.modules.base.Base):
             """add a job type for raw xml"""
@@ -192,11 +205,7 @@ class JenkinsJobManager:
         xml_jobs = xml_job_generator.generateXML(job_data_list)
         jobs = self.jobs
         for xml_job in xml_jobs:
-            # extra normalization
-            if False:
-                formatted_xml_str = self.jenkins_format_xml(xml_job)
-            else:
-                formatted_xml_str = self.xml_dump(xml_job.xml)
+            formatted_xml_str = self.xml_dump(xml_job.xml)
             jobs[xml_job.name].after_xml = formatted_xml_str
 
         xml_views = xml_view_generator.generateXML(view_data_list)
@@ -268,6 +277,30 @@ class JenkinsJobManager:
             template.stream(raw_xml_jobs=xml_job_name_pairs).dump(fp)
         return missing
 
+    def validate_metadata(self):
+        ET = xml.etree.ElementTree
+        md_conf = self.config.metadata
+
+        def extract_md(job: XmlChange):
+            node = ET.fromstring(job.after_xml)
+            desc = node.find("./description")
+            if desc is None:
+                log.warning("No description in jenkins job %r??", job.name)
+                return {}
+            md = {
+                m.group(1): m.group(2)
+                for m in re.finditer(r"^([\w-]+):\s*([\w-]+)$", desc.text, flags=re.M)
+            }
+            return md
+
+        for job in self.jobs.values():
+            if job.after_xml is None:
+                continue
+            md = extract_md(job)
+            warnings = md_conf.validate(md)
+            for warning in warnings:
+                yield job.name, warning
+
     def plan_report(self):
         """report on changes about to be made"""
         template = jinja2.Template(
@@ -358,7 +391,8 @@ No changes.
                 raise RuntimeError(
                     f"Invalid changetype {changetype}(id={id(changetype)})"
                 )
-
-        msg = f"Changes applied. added={changecounts[CREATE]} updated={changecounts[UPDATE]} deleted={changecounts[DELETE]}"
-
-        return (changecounts, msg)
+        msg = (
+            f"Changes applied. added={changecounts[CREATE]} updated={changecounts[UPDATE]}"
+            f" deleted={changecounts[DELETE]}"
+        )
+        return changecounts, msg
